//*****************************************************************************
//      C++ functions for:
//              colour management system
//
//      Written by Stuart Prescott:
//              Written:                16/10/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Formatted I/O
//
//      Other Object Classes used:
//              template arrays
//              maths extensions
//              callback
//
//*****************************************************************************

extern "C" {
    #include <libsx.h>
}

#include "arraytem.h"
#include "callback.h"
#include "mathxtra.h"
#include "coldefs.h"

#define NUMREDS		64
#define NUMGREENS	64

//The cols are allocated thus:
// NUMREDS colours are allocated such that:
// R     goes from STARTPRIMR to STOPPRIMGR
// G & B go   from STARTSECR to STOPSECR
//
//similarly for the greens . . .

#define STARTPRIMG	100		//these are 0->255 RGB vals
#define STOPPRIMG	230
#define STARTSECG	0
#define STOPSECG	230

#define STARTPRIMR      170
#define STOPPRIMR	240
#define STARTSECR	0
#define STOPSECR	220


#define DEBUGPAL	0

ColourDefs *pal;
 
ColourDefs::ColourDefs()
{
    nrcol=NUMREDS-1;
    ngcol=NUMGREENS-1;

    rpal=new intarray1(0,nrcol);
    gpal=new intarray1(0,ngcol);

    int col;
    int del1, del2;
    int val1, val2;

    int *lrpal=new int[NUMREDS];
    int *lgpal=new int[NUMREDS];
    int *lbpal=new int[NUMREDS];

   // int lrpal[16], lgpal[16], lbpal[16];

    #if DEBUGPAL
        cout << "Setting Up GREEN Palettes\n";
    #endif

    del2 = INT( (STOPSECG  - STARTSECG ) / NUMGREENS);
    del1 = INT( (STOPPRIMG - STARTPRIMG) / NUMGREENS);
    val2 = STARTSECG;
    val1 = STARTPRIMG;
    for (col=0; col<=ngcol; ++col)
    {
        lrpal[col]= val2;
	lgpal[col]= val1;
 	lbpal[col]= val2;
	val1 += del1;
	val2 += del2; 
    }
    for (col=0; col<=ngcol; ++col)
    {
	#if DEBUGPAL
	    printf ("r= %d   g= %d   b= %d    ",
                        lrpal[col], lgpal[col], lbpal[col]);
	#endif
	gpal->set(col, GetPrivateColor());
        #if DEBUGPAL
	    printf ("Entry %d\n", gpal->get(col));
	#endif
	if (gpal->get(col)==-1)
	{
	    ColourError();
	    continue;
	}
	SetPrivateColor(gpal->get(col),
		        lrpal[col], lgpal[col], lbpal[col]);
    }

    #if (NUMREDS!=NUMGREENS)
	delete[] lrpal;
	delete[] lgpal;
	delete[] lbpal;
	lrpal=new int[NUMREDS];
        lgpal=new int[NUMREDS];
        lbpal=new int[NUMREDS];
    #endif

    #if DEBUGPAL
	cout << "Setting Up RED Palettes\n";
    #endif

    del2 = INT( (STOPSECR  - STARTSECR ) / NUMREDS);
    del1 = INT( (STOPPRIMR - STARTPRIMR) / NUMREDS);
    val2 = STARTSECR;
    val1 = STARTPRIMR;
    for (col=0; col<=nrcol; ++col)
    {
        lrpal[col]= val1;
        lgpal[col]= val2;
        lbpal[col]= val2;
        val1 += del1;
        val2 += del2;
    }


    for (col=0; col<=nrcol; ++col)
    {
	#if DEBUGPAL
	    printf ("r= %d   g= %d   b= %d    ",
                        lrpal[col], lgpal[col], lbpal[col]);
	#endif
	rpal->set(col, GetPrivateColor());
        #if DEBUGPAL
	    printf ("Entry %d\n", rpal->get(col));
	#endif
	SetPrivateColor(rpal->get(col),
		        lrpal[col], lgpal[col], lbpal[col]);
    }


    //A couple of miscellaneous colours to finish off . . .
    white = GetPrivateColor();
    if (white==-1)
    {
	ColourError();
	//return;
    }
    SetPrivateColor(white, 255, 255, 255 );
    #if DEBUGPAL
        printf ("white Entry %d\n", white);
    #endif

    black = GetPrivateColor();
    if (black==-1)
    {   
        ColourError();
       // return;
    }   
    SetPrivateColor(black, 0, 0, 0);
    #if DEBUGPAL
        printf ("black Entry %d\n", black);
    #endif

    delete[] lrpal;
    delete[] lgpal;
    delete[] lbpal;

}

ColourDefs::~ColourDefs()
{
    for (int i=0; i<=15; ++i)
    {
	FreePrivateColor(rpal->get(i));
        FreePrivateColor(gpal->get(i));
    }
    FreePrivateColor(white);
    FreePrivateColor(black);

    delete rpal;
    delete gpal;
}
