//*****************************************************************************
//      C++ functions for:
//		control panel
//
//      Written by Stuart Prescott:
//              Written:                10/10/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//              Formatted I/O
//
//      Other Object Classes used:
//              template arrays
//              colours
//              fonts
//              main program data structure
//              callbacks
//
//*****************************************************************************

extern "C" {
    #include <libsx.h>
}

#include <iostream.h>
#include <stdio.h>

#include "panel.h"
#include "main.h"
#include "callback.h"
#include "coldefs.h"
#include "fontdefs.h"

#define DEBUGSELECT	0
#define DEBUGMARKCELL	0

extern ColourDefs *pal;
extern FontDefs *fonts;

// *************************************************************************  
// *************************************************************************  
Panel::Panel(int x, int y, Widget parent, int where1, Widget from1,
                                           int where2, Widget from2,
					   MyProgram *data)
{
    // Sets up the control panel . . .
    AllData=data;

    draw_area = MakeDrawArea(x, y, PanelRedisplay, this);

    //SetForm(parent);

    SetWidgetPos(draw_area, where1, from1, where2, from2);


    //initialise the required variables
    numbuttons = 5;
    x1      = new intarray1(1, numbuttons); 
    y1	    = new intarray1(1, numbuttons); 
    x2      = new intarray1(1, numbuttons); 
    y2	    = new intarray1(1, numbuttons); 
    deltax  = new intarray1(1, numbuttons); 
    deltay  = new intarray1(1, numbuttons); 

    text    = new char*[numbuttons];
    alttext = new char*[1];

    //set up the text strings to be displayed
    text[0]	= "Show Wavefunction";
    alttext[0]	= "Show Density";
    text[1]	= "Select Orbital";
    text[2]	= "Quit!";
    text[3]	= "Info";
    text[4]	= "Reset View";

    //setup the placement of the buttons on the draw_area.
    int gutter =5;
    x1->set(1, gutter);
    x1->set(2, gutter);
    x1->set(3, 75);
    x1->set(4, gutter);
    x1->set(5, gutter);


    int cellheight  = 30;
    int thiscell    = gutter;
    y1->set(1, thiscell);
    y1->set(2, thiscell+= cellheight);
    y1->set(5, thiscell+= cellheight);
    y1->set(3, thiscell+= cellheight);
    y1->set(4, thiscell);

    deltax->set(1,140);
    deltax->set(2,140);
    deltax->set(3,70);
    deltax->set(4,70);
    deltax->set(5,140);

    deltay->set(1,cellheight);
    deltay->set(2,cellheight);
    deltay->set(3,cellheight);
    deltay->set(4,cellheight);
    deltay->set(5,cellheight);

    for (int butnum= 1; butnum<=numbuttons; ++butnum)
    {
	x2->set(butnum, x1->get(butnum) + deltax->get(butnum) );
	y2->set(butnum, y1->get(butnum) + deltay->get(butnum) );
    }
    InSeparateWin=FALSE;
}
// *************************************************************************
// *************************************************************************
Panel::~Panel()
{
    delete x1;
    delete y1;
    delete x2;
    delete y2;
    delete deltax;
    delete deltay;
}

// *************************************************************************  
// *************************************************************************  
void PanelRedisplay(Widget w, int width, int height, void *data)
{
    // Redisplay CB for the control panel draw_area

    Panel *me = (Panel *)data;

    SetBgColor(me->draw_area, pal->black);
    SetColor(pal->black);
    SetDrawMode (GXcopy);
    DrawFilledBox(0,0, width, height);

    SetButtonDownCB(me->draw_area, PanelMouseDownCB);

    me->DrawButtons();

    return;
}

// *************************************************************************  
// *************************************************************************  
void Panel::DrawButtons()  
{  
    // Draws the buttons in
    
    SetColor(pal->white);
    int	i, butnum; 
  
    SetWidgetFont(draw_area, fonts->button);  
    int fontheight= FontHeight(fonts->button);  
  
    for (butnum= 1; butnum<=numbuttons; ++butnum)
    {
	DrawBox (x1->get(butnum),	y1->get(butnum),  
        	 deltax->get(butnum),	deltay->get(butnum));  

	i= TextWidth (fonts->button, text[butnum-1]);  
	#if DEBUGDRAWBOX  
	    cout << "TextWidth i="<< i<<"\n";  
	#endif  

	#if DEBUGSELECTSTR  
	    cout << "text["<<butnum<<"]\n";  
	    cout <<  text[butnum-1]<<"\n";  
	#endif  

	DrawText (text[butnum-1], (x1->get(butnum)+x2->get(butnum)-i)/2,
			(y1->get(butnum)+y2->get(butnum)+fontheight)/2);
    }

    if (!InSeparateWin)
    {
        lastcell=0;  
        cell=0;  
    }
    else
    {
	SetDrawMode(SANE_XOR);
	InvButton(cell);
	SetDrawMode(GXcopy);
    } 
    return;
}  
  
// *************************************************************************  
// *************************************************************************  
int Panel::HitButton (int x, int y)  
{  
  
    // **** see where mouse is ****  
    int i= 1;  
  
    // loop thru all the boxes until mouse co-ord is found within . . .  
    while ( (i<=numbuttons) &&  
              (     (x1->get(i)>x) || (x2->get(i)<x) ||  
                    (y1->get(i)>y) || (y2->get(i)<y)      )  
          ) i++;  
  
    #if DEBUGSELECT  
	cout << "x1, y1 "<< x1->get(i) << ", "<< y1->get(i) << "\n"  
	     << "x2, y2 "<< x2->get(i) << ", "<< y2->get(i) << "\n"  
	     << "cell= "<<i <<"\n";  
    #endif  
  
    lastcell = cell;  
  
    cell = ( ( (i>=0)&&(i <=numbuttons) ) ? i : 0 );
    InCell = cell;
    return InCell;
}  
  
// *************************************************************************  
// *************************************************************************  
void Panel::MarkButton ()  
{  
    //**** highlight box ****  
    #if DEBUGMARKCELL  
	cout << lastcell << " " <<cell<<"\n";  
    #endif  
    if ( (lastcell) && (cell!=lastcell) )  
    {
	InvButton(lastcell);  
    }  
    if ((cell!=lastcell)&&(InCell))  
    {
        InvButton(cell);  
    }  
    return;  
}  

// *************************************************************************
// *************************************************************************
void Panel::InvButton(int num)
{
    // inverts a button

    DrawFilledBox (x1->get(num),      y1->get(num),
                   deltax->get(num),  deltay->get(num));
    return;
}

// *************************************************************************
// *************************************************************************
void Panel::response()
{
    // formulates the response to a mouse click event

    switch (cell)
    {
	case 1:
	{
            Button_functype(NULL, AllData);
	    break;
	}
	case 2:
	{
     	    Button_SetMode(NULL, AllData);
	    break;
	}
	case 3:
	{
	    Button_quit(NULL, AllData);
	    break;
	}
	case 4:
	{
	    InSeparateWin=TRUE;
	    NewHelpWin(AllData);
	    InSeparateWin=FALSE;
	    break;
	}
	case 5:
	{
	    Button_ResetView(NULL, AllData);
	    break;
	}
    }
    
    SetDrawArea(draw_area);
    SetDrawMode(SANE_XOR);
    InvButton(cell);		// remove button hi-lighting after job done

    if (cell==1)
    {
	AltLabel(cell);		// put alternate label in button
    }
    SetDrawMode(GXcopy);
    lastcell=0;
    cell=0;

    return;
}

// *************************************************************************
// *************************************************************************
void Panel::AltLabel(int butnum)
{
    // puts an alternate label in a button (for a mode toggle)

    SetWidgetFont(draw_area, fonts->button);
    int fontheight= FontHeight(fonts->button);

    SetDrawMode(GXcopy);
    SetColor(pal->black);
    InvButton(butnum);

    SetColor(pal->white);
    char *temp;
    temp		= text[butnum-1];
    text[butnum-1] 	= alttext[butnum-1];
    alttext[butnum-1] 	= temp;

    DrawBox (x1->get(butnum),       y1->get(butnum),
             deltax->get(butnum),   deltay->get(butnum));

    int i= TextWidth (fonts->button, text[butnum-1]);

    DrawText (text[butnum-1], (x1->get(butnum)+x2->get(butnum)-i)/2,
                              (y1->get(butnum)+y2->get(butnum)+fontheight)/2);
    return;
}

// *************************************************************************  
// *************************************************************************  
void PanelMouseDownCB(Widget w, int which, int x, int y, void *data)
{
    //sets up the control procedures for the buttons

    if (which != 1) return;
  
    Panel *me = (Panel *)data;

    SetDrawMode(SANE_XOR);  
    if (me->HitButton(x, y))  
    {  
        me->MarkButton();  
  
	SetButtonDownCB	    (me->draw_area, NULL);
	SetButtonUpCB	    (me->draw_area, PanelMouseUpCB);
	SetMouseMotionCB    (me->draw_area, PanelMouseMotionCB);
  
    }  
    return;  
}

// *************************************************************************  
// *************************************************************************  
void PanelMouseMotionCB(Widget w, int x, int y, void *data)
{
    //repsonds to mouse movement over the buttons

    Panel *me = (Panel *)data;
  
    me->HitButton(x, y);  
    me->MarkButton();  
    return;  
}

// *************************************************************************  
// *************************************************************************  
void PanelMouseUpCB(Widget w, int which, int x, int y, void *data)
{
    // when button raised, (if valid response) act accodingly

    Panel *me = (Panel *)data;

    if (me->HitButton(x, y))  
    {  
	SetButtonDownCB	(me->draw_area, PanelMouseDownCB);
	SetButtonUpCB	(me->draw_area, NULL);
	SetMouseMotionCB	(me->draw_area, NULL);
	//SetDrawMode(GXcopy);  
        me->response(); 
    }  
    else  
    {  
	SetButtonDownCB	    (me->draw_area, PanelMouseDownCB);
	SetButtonUpCB	    (me->draw_area, NULL);
	SetMouseMotionCB    (me->draw_area, NULL);
 	me->MarkButton();  
	SetDrawMode(GXcopy);	    //is this necessary???  
    }  
    return;  
}
